﻿using GrapeCity.ActiveReports.Aspnetcore.Viewer;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;

namespace AR16JSViewerMiddleware
{
    public class Startup
    {
        public static string EmbeddedReportsPrefix = "AR16JSViewerMiddleware.Reports";
        private static Regex RenderRequestRegex = new Regex("/api/reporting/reports/(.+)/render");

        // This method gets called by the runtime. Use this method to add services to the container.
        public void ConfigureServices(IServiceCollection services)
        {
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);

            services
                .AddLogging(config =>
                {
                    // Disable the default logging configuration
                    config.ClearProviders();

                    // Enable logging for debug mode only
                    if (Environment.GetEnvironmentVariable("ASPNETCORE_ENVIRONMENT") == Environments.Development)
                    {
                        config.AddConsole();
                    }
                })
                .AddReporting()
                .AddMvc(option => option.EnableEndpointRouting = false);
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
            public void Configure(IApplicationBuilder app, IWebHostEnvironment env, ILogger<Startup> logger)
            {
                if (env.IsDevelopment())
                {
                    app.UseDeveloperExceptionPage();
                }
                app.UseWhen(context => RenderRequestRegex.IsMatch(context.Request.Path.Value), (app) =>
                {
                    app.Use(async (context, next) =>
                    {
                        // extract report name from the request
                        var reportName = RenderRequestRegex.Match(context.Request.Path.Value).Groups[1].Value;
                        logger.LogWarning($"Report {reportName} has been requested at {DateTime.Now:dd-MM-yyyy HH:mm:ss}");
                        context.Request.EnableBuffering();
                        using (var reader = new StreamReader(context.Request.Body, encoding: Encoding.UTF8, detectEncodingFromByteOrderMarks: false, bufferSize: 1024, leaveOpen: true))
                        {
                            var body = await reader.ReadToEndAsync();
                            // extract parameters values from the body request
                            var renderRequest = Newtonsoft.Json.JsonConvert.DeserializeObject<RenderRequest>(body);
                            foreach (var kv in renderRequest.Parameters)
                            {
                                logger.LogWarning($"Parameter {kv.Key} has values {string.Join(",", kv.Value)}");
                            }
                            context.Request.Body.Position = 0;
                        }
                        await next.Invoke();
                    });
                });
                app.UseReporting(settings =>
                {
                    settings.UseEmbeddedTemplates(EmbeddedReportsPrefix, Assembly.GetAssembly(GetType()));
                    settings.UseCompression = true;
                });

                app.UseMvc();
            }
    }
    class RenderRequest { public Dictionary<string, object[]> Parameters { get; set; } }
}
